#!/usr/bin/python

## expendable

## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
if __name__ == '__main__':
    sys.path.append (os.environ.get ('top_srcdir', '.'))

from gettext import gettext as _
import gtk.glade
import account

name=_("Extra income")
description=_("Additional income every month.")

class Model(account.Model):
    def __init__ (self, amount):
        account.Model.__init__ (self)
        self.amount = amount

    def cash_available (self):
        return self.amount

    def cash_available_next_month (self):
        return self.amount

    def iter (self):
        if self.parent == None:
            return False

        amount = -self.parent.adjust (self.amount, self)
        self.model_data.append( { 'month': self.per,
                                  'amount': self.amount} )
        return False

class Data(account.Data):
    required_tags = ['amount']
    glade = "gain.glade"

    def __init__ (self):
        account.Data.__init__ (self)
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        data = {}
        for name in ['amount']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value ()
        self.data = data

    def is_leaf_node (self):
        return True

    def read (self, xml):
        data = {}
        
        for child in xml.getchildren ():
            # Floats
            if child.tag in ['amount']:
                data[child.tag] = float (child.text)
            else:
                print "Not understood: %s" % child.tag

        for required in self.required_tags:
            if not data.has_key (required):
                raise RuntimeError, "Required tags: %s" % self.required_tags

        self.data = data
        self._set_last_known_good ()

    def write (self):
        xml = "<gain>\n"
        for var in self.required_tags:
            value = self.data[var]
            if type (value) == float:
                value = "%.4f" % value
            xml += "  <%s>%s</%s>\n" % (var, value, var)
        xml += "</gain>\n"
        return xml

    def ui (self, variables_vbox, results_vbox):
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        w = gld.get_widget ('variables_table')
        w.reparent (variables_vbox)
        for var in self.required_tags:
            widget = gld.get_widget (var)
            widget.set_value (self.data[var])
            widget.connect ("value-changed", self.variable_changed, var)

    def variable_changed (self, widget, var):
        self.data[var] = widget.get_value ()
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good[var])

    def get_model (self):
        return Model (**self.data)

if __name__ == '__main__':
    from pprint import *
    from xml.etree.ElementTree import XML
    d = Data ()
    w = d.write ()
    xml = XML (w)
    d.read (xml)

    import savings
    s = savings.Model (rate=0,
                       balance=0,
                       taxrate=0)
    gain = Model (amount=200)
    gain.set_delay (3)
    s.add_model (gain)
    s.run (nper=6)
    data = s.get_model_data ()
    if abs (data[5]['balance'] - 600.00) > 0.01:
        raise RuntimeError, "Incorrect amount gained"

