#!/usr/bin/python

## expendable

## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
if __name__ == '__main__':
    sys.path.append (os.environ.get ('top_srcdir', '.'))

from gettext import gettext as _
import gtk.glade
import locale
import account

name=_("Regular Saver")
description=_("Fixed term savings account with required monthly "
              "contributions and taxable interest.")

class Model(account.Model):
    def __init__ (self, rate, taxrate, amount, minamount, term):
        account.Model.__init__ (self)
        self.rate = rate
        self.taxrate = taxrate
        self.mrate = pow (1 + self.rate, 1 / 12.0) - 1
        self.amount = amount
        self.minamount = minamount
        self.term = term
        self.balance = 0
        self.gross = 0
        self.net = 0

    def cash_available (self):
        cash_available = account.Model.cash_available (self)
        if self.per > self.term:
            cash_available += self.balance
        else:
            contrib = self.parent.test_adjust (-self.amount, self)
            if contrib == self.amount:
                cash_available -= self.amount
            else:
                cash_available -= self.minamount

        return cash_available

    def cash_available_next_month (self):
        liabilities = -self.minamount
        if self.per >= self.term:
            liabilities = 0.0

        return liabilities

    def append_data (self, month, contrib=0.0, gross=0.0, net=0.0, tax=0.0):
        self.model_data.append( { 'month': month,
                                  'contrib': contrib,
                                  'balance': self.balance,
                                  'totalgross': self.gross,
                                  'totalnet': self.net,
                                  'totaltax': self.gross - self.net,
                                  'gross': gross,
                                  'net': net,
                                  'tax': tax } )

    def iter (self):
        contrib_amount = self.amount
        if self.per < self.term:
            # Take the monthly contribution from our parent account.
            if self.parent != None:
                try:
                    contrib = self.parent.test_adjust (-contrib_amount, self)
                    if contrib < contrib_amount:
                        contrib_amount = self.minamount
                except RuntimeError:
                    # Can't use test_adjust.  Fall back to actually
                    # transferring the amount and then transferring it
                    # back on failure.
                    print "no test available"

                contrib = self.parent.adjust (-contrib_amount, self)
                if (contrib < contrib_amount and
                    contrib_amount != self.minamount):
                    contrib = self.parent.adjust (contrib_amount, self)
                    contrib_amount = self.minamount
                    contrib = self.parent.adjust (-contrib_amount, self)

                if contrib < contrib_amount:
                    self.parent.adjust (contrib_amount, self)
                    raise account.ModelError, (self, "Not enough contribution")
            else:
                # or just assume it was available in the past.
                contrib = self.amount
        else:
            contrib = 0

        gross = int (100 * self.balance * self.mrate) * 0.01
        tax = int (100 * gross * self.taxrate) * 0.01
        net = gross - tax
        self.model_data.append( { 'month': self.per,
                                  'contrib': contrib,
                                  'balance': self.balance,
                                  'totalgross': self.gross,
                                  'totalnet': self.net,
                                  'totaltax': self.gross - self.net,
                                  'gross': gross,
                                  'net': net,
                                  'tax': tax } )

        self.balance += net + contrib
        self.gross += gross
        self.net += net
        if self.per == self.term:
            if self.parent == None:
                raise account.ModelError, (self, "Finishes before model starts")
            self.parent.adjust (self.balance, self)
            self.balance = 0.0
            return True

        return False

    def get_datasets (self):
        model_data = self.model_data
        rdata = map (lambda x: (x['month'], x['balance']), model_data)
        return [{'name': self.account.get_name (),
                 'data': rdata}]

class Data(account.Data):
    required_tags = ['rate', 'taxrate', 'amount', 'minamount', 'term']
    glade = "regsave.glade"

    def __init__ (self):
        account.Data.__init__ (self)
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        data = {}
        for name in ['rate', 'taxrate']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value () / 100.0

        for name in ['amount', 'minamount', 'term']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value ()

        data['term'] = int (data['term'])
        self.data = data

    def is_leaf_node (self):
        return True

    def read (self, xml):
        data = {}
        for child in xml.getchildren ():
            # Floats
            if child.tag in ['rate',
                             'taxrate',
                             'amount',
                             'minamount']:
                data[child.tag] = float (child.text)
            elif child.tag in ['term']:
                # Ints
                data[child.tag] = int (child.text)
            else:
                print "Not understood: %s" % child.tag

        for required in self.required_tags:
            if not data.has_key (required):
                raise RuntimeError, "Required tags: %s" % self.required_tags

        self.data = data
        self._set_last_known_good ()

    def write (self):
        xml = "<regsave>\n"
        for var in self.required_tags:
            value = self.data[var]
            if type (value) == float:
                value = "%.4f" % value
            xml += "  <%s>%s</%s>\n" % (var, value, var)
        xml += "</regsave>\n"
        return xml

    def ui (self, variables_vbox, results_vbox):
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        w = gld.get_widget ('variables_table')
        w.reparent (variables_vbox)
        for var in ['rate', 'taxrate']:
            widget = gld.get_widget (var)
            widget.set_value (self.data[var] * 100.0)
            widget.connect ("value-changed", self.rate_changed, var)
        for var in ['amount', 'minamount', 'term']:
            widget = gld.get_widget (var)
            widget.set_value (self.data[var])
            widget.connect ("value-changed", self.variable_changed, var)

        w = gld.get_widget ('results_hbox')
        w.reparent (results_vbox)
        self.result_widget = {}
        for name in ['fbalance', 'totalnet']:
            self.result_widget[name] = gld.get_widget (name)

    def rate_changed (self, widget, name):
        self.data[name] = widget.get_value () / 100.0
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good[name] * 100)

    def variable_changed (self, widget, name):
        value = widget.get_value ()
        if name == 'term':
            value = int (value)
        self.data[name] = value
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good[name])

    def update_results (self, modeldata):
        fbalance = self.result_widget['fbalance']
        totalnet = self.result_widget['totalnet']
        n = len (modeldata)
        if n > 0:
            last = modeldata[n - 1]
            fbalance.set_text (locale.currency (self.data['amount'] +
                                                last['totalnet'], True, True))
            totalnet.set_text (locale.currency (last['totalnet'], True, True))
        else:
            fbalance.set_text ('')
            totalnet.set_text ('')

    def get_model (self):
        return Model (**self.data)

if __name__ == '__main__':
    from pprint import *
    from xml.etree.ElementTree import XML
    d = Data ()
    w = d.write ()
    xml = XML (w)
    d.read (xml)

    import savings
    s = savings.Model (rate=0,
                       balance=6000,
                       taxrate=0)
    regsave = Model (rate=0.10,
                     taxrate=0.20,
                     amount=500,
                     minamount=25,
                     term=12)
    s.add_model (regsave)
    s.run (nper=14)
    data = regsave.get_model_data ()
    #pprint (data)
    data = s.get_model_data ()
    #pprint (data)
    if abs (data[12]['balance'] - 6254.70) > 0.01:
        raise RuntimeError, "Regular saver incorrect interest"
