#!/usr/bin/python

## expendable

## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
if __name__ == '__main__':
    sys.path.append (os.environ.get ('top_srcdir', '.'))

from gettext import gettext as _
import gtk.glade
import account

name=_("Spending")
description=_("A regular monthly withdrawal (for example, a "
              "shortfall due to spending).")

class Model(account.Model):
    def __init__ (self, amount):
        account.Model.__init__ (self)
        self.amount = amount
        self.total = 0

    def cash_available (self):
        return -self.amount

    def cash_available_next_month (self):
        return -self.amount

    def iter (self):
        if self.parent == None:
            return False

        amount = self.parent.adjust (-self.amount, self)
        if amount < self.amount:
            raise account.ModelError, (self, "Not enough money to spend")

        self.total += self.amount
        self.model_data.append( { 'month': self.per,
                                  'total': self.total} )
        return False

class Data(account.Data):
    required_tags = ['amount']
    glade = "spend.glade"

    def __init__ (self):
        account.Data.__init__ (self)
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        data = {}
        for name in ['amount']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value ()
        self.data = data

    def is_leaf_node (self):
        return True

    def read (self, xml):
        data = {}
        for child in xml.getchildren ():
            # Floats
            if child.tag in ['amount']:
                data[child.tag] = float (child.text)
            else:
                print "Not understood: %s" % child.tag

        for required in self.required_tags:
            if not data.has_key (required):
                raise RuntimeError, "Required tags: %s" % self.required_tags

        self.data = data
        self._set_last_known_good ()

    def write (self):
        xml = "<spend>\n"
        for var in self.required_tags:
            value = self.data[var]
            if type (value) == float:
                value = "%.4f" % value
            xml += "  <%s>%s</%s>\n" % (var, value, var)
        xml += "</spend>\n"
        return xml

    def ui (self, variables_vbox, results_vbox):
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        w = gld.get_widget ('variables_table')
        w.reparent (variables_vbox)
        widget = gld.get_widget ('amount')
        widget.set_value (self.data['amount'])
        widget.connect ("value-changed", self.amount_changed)

    def amount_changed (self, widget):
        self.data['amount'] = widget.get_value ()
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good['amount'])

    def get_model (self):
        return Model (**self.data)

if __name__ == '__main__':
    from pprint import *
    from xml.etree.ElementTree import XML
    d = Data ()
    w = d.write ()
    xml = XML (w)
    d.read (xml)

    import savings
    s = savings.Model (rate=0, taxrate=0, balance=1000)
    spend = Model (amount=100)
    s.add_model (spend)
    s.run (nper=10)
    data = s.get_model_data ()
    #pprint (data)
    if abs (data[9]['balance']) > 0.01:
        raise RuntimeError, "Not spent enough"
