#!/usr/bin/python

## expendable

## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
if __name__ == '__main__':
    sys.path.append (os.environ.get ('top_srcdir', '.'))

from gettext import gettext as _
import gtk.glade
import locale
import account

name=_("Credit Card (Promotional Rate)")
description=_("Credit card with a time-limited promotional rate "
              "for purchases.")

class Model(account.Model):
    def __init__ (self, balance, limit, rate, term,
                  minimum_payment_pc, minimum_payment_base):
        account.Model.__init__ (self)
        self.balance = balance
        self.limit = limit
        self.rate = rate
        self.mrate = rate / 12
        self.term = term
        self.minimum_payment_pc = minimum_payment_pc
        self.minimum_payment_base = minimum_payment_base
        self.totalint = 0.0
        self.recent_transactions = 0.0

    def adjust (self, amount, by_account):
        if self.per < 0 or self.per >= self.term:
            # We haven't started (or are finished); try our parent.
            if by_account == self.parent:
                return self._adjust_recursive (amount, self)
            else:
                if self.parent == None:
                    return -amount

                return self.parent.adjust (amount, by_account)

        if by_account.__module__ == self.__module__:
            # Can't repay a credit card with a credit card.
            if self.parent == None:
                return -amount
            return self.parent.adjust (amount, by_account)

        adjustment = amount
        if self.balance + self.recent_transactions - amount > self.limit:
            adjustment = -(self.limit - self.balance - self.recent_transactions)
            self.recent_transactions -= adjustment
            amount -= adjustment
            if by_account in self.models:
                # One of the accounts linked to this is trying to
                # spend money.  If our limit prevents that, pass it on
                # to the parent.
                if self.parent != None:
                    part = self.parent.adjust (amount, by_account)
                    adjustment -= part
                else:
                    adjustment = amount
            else:
                for model in self.models:
                    try:
                        part = model.adjust (amount, self)
                    except RuntimeError:
                        # Not implemented by that model.
                        continue

                    amount += part
                    adjustment -= part

            if amount > 0.01:
                raise account.ModelError, (self, "Credit limit exceeded")
        else:
            self.recent_transactions -= adjustment

        return -adjustment

    def cash_available (self):
        cash_available = account.Model.cash_available (self)
        cash_available -= self.balance + self.recent_transactions
        return cash_available

    def cash_available_next_month (self):
        cash_available = account.Model.cash_available_next_month (self)
        if self.per >= self.term:
            # On the last month we want to pay the whole lot off.
            cash_available += -(self.balance + self.recent_transactions)
        else:
            # Before that it's just the minimum payment.
            min_pmt = self.balance * self.minimum_payment_pc
            if min_pmt < self.minimum_payment_base:
                min_pmt = self.minimum_payment_base
                if min_pmt > self.balance:
                    min_pmt = self.balance
            cash_available += -min_pmt

        return cash_available

    def iter (self):
        interest = int (100 * self.balance * self.mrate) * 0.01
        self.totalint += interest
        self.balance += self.recent_transactions
        self.recent_transactions = 0.0

        min_pmt = int (100 * self.balance * self.minimum_payment_pc) * 0.01
        if min_pmt < self.minimum_payment_base:
            min_pmt = self.minimum_payment_base
            if min_pmt > self.balance:
                min_pmt = self.balance

        if self.per == self.term:
            if self.parent == None:
                # We are the top-level account.
                pmt = self.balance
            else:
                pmt = self.parent.adjust (-self.balance, self)

            if pmt < self.balance:
                raise account.ModelError, (self, "Promotional period has ended")
            self.balance -= pmt
        else:
            if min_pmt > self.balance:
                min_pmt = self.balance
            if self.parent == None:
                # We are the top-level account.
                pmt = min_pmt
            else:
                pmt = self.parent.adjust (-min_pmt, self)
                if pmt < min_pmt:
                    raise account.ModelError, (self,
                                               "Unable to meet min payment")
            self.balance -= pmt

        self.model_data.append( { 'month': self.per,
                                  'balance': self.balance,
                                  'interest': interest,
                                  'total interest': self.totalint,
                                  'min_pmt': min_pmt } )

        self.balance += interest
        return False

    def get_datasets (self):
        model_data = self.model_data
        cdata = map (lambda x: (x['month'],
                                x['balance']), model_data)
        return [{'name': self.account.get_name (),
                 'data': cdata }]

class Data(account.Data):
    required_tags = ['balance', 'limit', 'rate', 'term',
                     'minimum_payment_pc', 'minimum_payment_base']
    glade = "stooze.glade"

    def __init__ (self):
        account.Data.__init__ (self)
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        data = {}
        for name in ['rate', 'minimum_payment_pc']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value () / 100.0

        for name in ['balance', 'limit', 'term', 'minimum_payment_base']:
            widget = gld.get_widget (name)
            data[name] = widget.get_value ()

        data['term'] = int (data['term'])
        self.data = data

    def read (self, xml):
        data = {}
        for child in xml.getchildren ():
            # Floats
            if child.tag in ['balance',
                             'limit',
                             'rate',
                             'minimum_payment_pc',
                             'minimum_payment_base']:
                data[child.tag] = float (child.text)
            elif child.tag in ['term']:
                # Ints
                data[child.tag] = int (child.text)
            else:
                print "Not understood: %s" % child.tag

        for required in self.required_tags:
            if not data.has_key (required):
                raise RuntimeError, "Required tags: %s" % self.required_tags

        self.data = data
        self._set_last_known_good ()

    def write (self):
        xml = "<stooze>\n"
        for var in self.required_tags:
            xml += "  <%s>%s</%s>\n" % (var, self.data[var], var)
        xml += "</stooze>\n"
        return xml

    def ui (self, variables_vbox, results_vbox):
        for dir in sys.path:
            f = os.path.join (dir, self.glade)
            if os.access (f, os.R_OK):
                gld = gtk.glade.XML (f)
                break

        w = gld.get_widget ('variables_table')
        w.reparent (variables_vbox)
        for var in ['rate', 'minimum_payment_pc']:
            widget = gld.get_widget (var)
            widget.set_value (self.data[var] * 100.0)
            widget.connect ("value-changed", self.rate_changed, var)
        for var in ['balance', 'limit', 'term', 'minimum_payment_base']:
            widget = gld.get_widget (var)
            widget.set_value (self.data[var])
            widget.connect ("value-changed", self.variable_changed, var)

        w = gld.get_widget ('results_hbox')
        w.reparent (results_vbox)
        self.result_widget = {}
        for name in ['totalint']:
            self.result_widget[name] = gld.get_widget (name)

    def rate_changed (self, widget, name):
        self.data[name] = widget.get_value () / 100.0
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good[name] * 100)

    def variable_changed (self, widget, name):
        value = widget.get_value ()
        if name == 'term':
            value = int (value)
        self.data[name] = value
        try:
            self.redisplay ()
        except Exception, e:
            print e
            widget.set_value (self.last_known_good[name])

    def update_results (self, modeldata):
        totalint = self.result_widget['totalint']
        n = len (modeldata)
        if n > 0:
            last = modeldata[n - 1]
            totalint.set_text (locale.currency (last['total interest'],
                                                True, True))
        else:
            totalint.set_text ('')

    def get_model (self):
        return Model (**self.data)

if __name__ == '__main__':
    from pprint import *
    from xml.etree.ElementTree import XML
    d = Data ()
    w = d.write ()
    xml = XML (w)
    d.read (xml)

    import savings, spend
    s = savings.Model (rate=0.06,
                       balance=1000,
                       taxrate=0)
    stooze = Model (balance=0.0,
                    limit=2000.0,
                    rate=0.0,
                    term=10,
                    minimum_payment_pc=0.03,
                    minimum_payment_base=5.0)
    s.add_model (stooze)
    spend = spend.Model (amount=50.0)
    stooze.add_model (spend)
    try:
        s.run (nper=14)
    except Exception, e:
        print e
    data = s.get_model_data ()
    #pprint (data)
    if abs (data[12]['balance'] - 403.23) > 4.01:
        raise RuntimeError, "Stooze incorrect interest (%s)" % data[12]['balance']

    # Make sure the first minimum payment is handled correctly.
    s = savings.Model (rate=0.06,
                       balance=1000,
                       taxrate=0)
    stooze = Model (balance=4000.0,
                    limit=4000.0,
                    rate=0.0,
                    term=10,
                    minimum_payment_pc = 0.03,
                    minimum_payment_base = 5.0)
    s.add_model (stooze)
    s.run (nper=1)
    data = s.get_model_data ()
    if abs (data[0]['balance'] - 880.0) > 0.01:
        raise RuntimeError, "No minimum payment at month 0"
